% Copyright 2006 by Till Tantau
%
% This file may be distributed and/or modified
%
% 1. under the LaTeX Project Public License and/or
% 2. under the GNU Public License.
%
% See the file doc/generic/pgf/licenses/LICENSE for more details.

\ProvidesFileRCS $Header: /cvsroot/pgf/pgf/generic/pgf/systemlayer/pgfsys-xetex.def,v 1.3 2010/02/16 10:56:09 tantau Exp $

% Driver commands for dvipdfm

%
% Load common pdf and pdf in dvi commands:
%
\input pgfsys-common-pdf.def
\input pgfsys-common-pdf-via-dvi.def

%
% dvipdfmx-specific stuff:
%

\def\pgfsys@invoke#1{\special{pdf:literal direct #1 }}


\ifx\XeTeXversion\@undefined
  \PackageError{pgfsys}{XeTeX is not running, but it should. Somehow
    the wrong driver got selected}{}
\fi


%
% Position tracking always on
%
\pgf@sys@pdf@dvi@tracker

% 
% Page size stuff
% 
\def\pgfsys@papersize#1#2{%
 \pdfpageheight#2%
 \pdfpagewidth#1}


\endinput

%%% Local Variables: 
%%% mode: latex
%%% End: 
